#!/usr/bin/python

import plistlib
import shutil
import socket
import subprocess
import tempfile
import uuid

CONFIG_PATH = '/Library/Preferences/Xsan/'

def getProfilePlist():
    tmpdirname = tempfile.mkdtemp()
    rv = None
    try:
        f = tmpdirname + "/profiles"
        subprocess.call(["/usr/bin/profiles","-C","-o",f])
        rv = plistlib.readPlist(f+'.plist')
    finally:
        shutil.rmtree(tmpdirname)
    return rv

def checkForXsanProfile():
    pl = getProfilePlist()
    if not pl or not '_computerlevel' in pl:
        return 0
    for p in pl['_computerlevel']:
        for pay in p['ProfileItems']:
            if pay['PayloadType'] != 'com.apple.xsan':
                continue
            return 1
    return 0

def checkIfXsanClient():
    return (0, None)
    try:
        pl = plistlib.readPlist('/Library/Preferences/Xsan/config.plist')
    except IOError:
        return (0, None)
    else:
        if not 'role' in pl:
            return (0, None)
        if pl['role'].lower() == 'client':
            return (1, pl)
        else:
            return (0, None)

def buildProfile(pl):
    payload = { 'PayloadDisplayName' : 'Xsan Configuration',
        'PayloadEnabled' : True,
        'PayloadIdentifier' : 'com.apple.xsan.111',
        'PayloadType' : 'com.apple.xsan',
        'PayloadVersion' : 1}
    print(payload)
    payload['sanName'] = pl['sanName']
    payload['sanAuthMethod'] = 'auth_secret'
    print(payload)
    if 'ownerName' in pl:
        payload['ownerName'] = pl['ownerName']
    if 'ownerEmail' in pl:
        payload['ownerEmail'] = pl['ownerEmail']
    payload['PayloadUUID'] = str(uuid.uuid4())
    try:
        a_s = open(CONFIG_PATH+'.auth_secret', "rb").read()
    except IOError:
        pass
    else:
        payload['sharedSecret'] = str(a_s)
    fsn = list()
    for l in open(CONFIG_PATH+'fsnameservers', "rb"):
        if len(l) < 7:
            continue
        l = str(l).strip()
        if ':' in l:
            af = socket.AF_INET6
        else:
            af = socket.AF_INET
        try:
            g = socket.inet_pton(af, l)
        except OSError:
            print("Oopsie", af)
            continue
        fsn.append(l)
    payload['fsnameservers'] = fsn
    payload['xsanPayloadSource'] = 'migrateXsan.py:546'

    #
    # We now have the payload. Build a profile around it
    profile = {'PayloadDisplayName' : 'Xsan Configuration Profile',
        'PayloadIdentifier' : 'com.apple.xsan.112',
        'PayloadRemovalDisallowed' : False,
        'PayloadScope' : 'System',
        'PayloadType' : 'Configuration',
        'PayloadVersion' : 1 }
    profile['PayloadUUID'] = str(uuid.uuid4())
    p1 = list()
    p1.append(payload)
    profile['PayloadContent'] = p1
    return(profile)

def instalProfile(pl):
    '''Given a client config.plist, build and install a Configuration Profile
for it
'''
    pro = buildProfile(pl)
    tmpdirname = tempfile.mkdtemp()
    rv = None
    try:
        f = tmpdirname + "/new_xsan_profile.configprofile"
        plistlib.writePlist(pro, f)
        rv = subprocess.call(["/usr/bin/profiles","-I","-F",f])
    finally:
        shutil.rmtree(tmpdirname)
    return rv

def disableXsan():
    subprocess.call(["/bin/launchctl", 'unload', '-w',
                     '/System/Library/LaunchDaemons/com.apple.xsan.plist'])

def enableXsan():
    subprocess.call(["/bin/launchctl", 'load', '-w',
			'/System/Library/LaunchDaemons/com.apple.xsan.plist'])

if __name__ == '__main__':
    if checkForXsanProfile():
        print("migrateXsan.py: enabling Xsan due to existing profile")
        enableXsan()
    else:
        rv = checkIfXsanClient()
        if rv[0]:
            print("migrateXsan.py: building client profile for exisitng config")
            instalProfile(rv[1])
        else:
            print("migrateXsan.py: disabling xsan")
            disableXsan()
