#!/bin/bash

# MCPF Recovery Creator. Copyrights (c) @MLforAll. All rights reserved.

# This script must be run from the Temporary Recovery (TempRec)
# Temporary Recovery : A Recovery that is stored on a normal partition

### Funcs declarations ###

createRecWithBSZO () {
	/swap/.IABootFiles/dmtest ensureRecoveryPartition $1 "$2" 0 0 "/swap/.IABootFiles/BaseSystem.chunklist" > /swap/.IABootFiles/dmtestPipeline.mcpf || echo "err"
}
replaceRecBSbyBSZO () {
	rm /swap/.IABootFiles/RecHD/com.apple.recovery.boot/BaseSystem.dmg
	cp /swap/.IABootFiles/BSZO.dmg /swap/.IABootFiles/RecHD/com.apple.recovery.boot/BaseSystem.dmg
}
createDestInfoPlist () {
	if [ -f /swap/.IABootFiles/destInfo.mcpf.plist ]; then
		rm /swap/.IABootFiles/destInfo.mcpf.plist
	fi
	diskutil info -plist /swap/ > /swap/.IABootFiles/destInfo.mcpf.plist
}

### Getting some infos on the dest volume ###

if [ -f /ec.inst ]; then csrutil disable || echo "csrutil failed! We continue the Recovery creation but the user might have issues related to SIP!" ; fi

createDestInfoPlist
destNode=$(defaults read /swap/.IABootFiles/destInfo.mcpf.plist DeviceNode)

if [ -f /smallRecovery.mcpfoption -o "$1" == "tryAgainSmallRec" ]; then

	defaults read /swap/.IABootFiles/destInfo.mcpf.plist RecoveryDeviceIdentifier

	createRecWithBSZO $destNode "/swap/.IABootFiles/origBS.dmg"
	/System/Installation/CDIS/osascript /swap/.IABootFiles/mountRec.scpt "stdout" "/swap/.IABootFiles/RecHD"
	if [ ! -f /swap/.IABootFiles/RecHD/com.apple.recovery.boot/boot.efi ]; then exit 1 ; fi

	diskutil info -plist "/swap/.IABootFiles/RecHD" > /swap/.IABootFiles/rechdInfos.mcpf.plist
	recNode=$(defaults read /swap/.IABootFiles/rechdInfos.mcpf.plist DeviceNode)
	totalRecSize=$(defaults read /swap/.IABootFiles/rechdInfos.mcpf.plist TotalSize)
	if (( $totalRecSize > 659998336 )); then
		diskutil resizeVolume $recNode 650M
		diskutil unmount $recNode
		mkdir "/swap/.IABootFiles/RecHD"
		mount_hfs $recNode "/swap/.IABootFiles/RecHD"
	fi
	rm /swap/.IABootFiles/rechdInfos.mcpf.plist


	replaceRecBSbyBSZO
	kextcache -prelinked-kernel /swap/.IABootFiles/kc -z -K /swap/.IABootFiles/osxkern -volume-root "/" "/System/Library/Extensions"

	if [ -f /swap/.IABootFiles/kc ]; then
		if [ -f "/swap/.IABootFiles/RecHD/com.apple.recovery.boot/kernelcache" ]; then kcache="/swap/.IABootFiles/RecHD/com.apple.recovery.boot/kernelcache" ; else kcache="/swap/.IABootFiles/RecHD/com.apple.recovery.boot/prelinkedkernel" ; fi
		rm "$kcache"
		cp /swap/.IABootFiles/kc "/swap/.IABootFiles/RecHD/com.apple.recovery.boot/kernelcache"
	else
		echo "kc not found!"
	fi

else

	createRecWithBSZO $destNode "/swap/.IABootFiles/BSZO.dmg"
	/System/Installation/CDIS/osascript /swap/.IABootFiles/mountRec.scpt "stdout" "/swap/.IABootFiles/RecHD"

	if [ ! -f /swap/.IABootFiles/RecHD/com.apple.recovery.boot/boot.efi ]; then
		createDestInfoPlist
		destRecNode=$(defaults read /swap/.IABootFiles/destInfo.mcpf.plist RecoveryDeviceIdentifier)
		/System/Installation/CDIS/osascript /swap/.IABootFiles/mountRec.scpt "/dev/$destRecNode" "/swap/.IABootFiles/RecHD"
	fi

	if [ ! -f /swap/.IABootFiles/RecHD/com.apple.recovery.boot/boot.efi ]; then
		# The Recovery creation failed!
		if [ -f /swap/.IABootFiles/origBS.dmg ]; then
			#createRecWithBSZO $destNode "/swap/.IABootFiles/origBS.dmg"
			#/System/Installation/CDIS/osascript /swap/.IABootFiles/mountRec.scpt "stdout" "/swap/.IABootFiles/RecHD"
			#replaceRecBSbyBSZO
			touch /swap/.IABootFiles/errorRecovery.mcpf
			exit 1
		else
			echo "Unable to find the file pointed by origBS.dmg!"
			exit 1
		fi
	else
		dmgRecSize=$(stat -f%z "/swap/.IABootFiles/RecHD/com.apple.recovery.boot/BaseSystem.dmg")
		dmgNewSize=$(stat -f%z "/swap/.IABootFiles/BSZO.dmg")
		if [ dmgRecSize == dmgNewSize ]; then
			replaceRecBSbyBSZO
		else
			exit 1
		fi
	fi

fi

### In case of weird behavior, check if the RecHD's mountpoint folder isn't empty ###

if [ ! -f /swap/.IABootFiles/RecHD/com.apple.recovery.boot/boot.efi ]; then
	rm -R /swap/.IABootFiles/RecHD
	exit 1 # Recovery is probably not mounted...
fi

### Managing some files ###

rm "/swap/.IABootFiles/BSZO.dmg"

rm /swap/.IABootFiles/RecHD/com.apple.recovery.boot/PlatformSupport.plist	
cp /swap/.IABootFiles/BootLogo.png /swap/.IABootFiles/RecHD
cp "/swap/System/Library/CoreServices/uuid.mcpf" /swap/.IABootFiles/RecHD/com.apple.recovery.boot
	
### Changing boot-args ###

if [ -f /ec.inst -o -f /yos.inst ]; then
	kflags="rp=file:///com.apple.recovery.boot/BaseSystem.dmg kext-dev-mode=1"
else
	kflags="rp=file:///com.apple.recovery.boot/BaseSystem.dmg"
fi

rm /swap/.IABootFiles/RecHD/com.apple.recovery.boot/com.apple.Boot.plist
cp /swap/.IABootFiles/com.apple.Boot.plist /swap/.IABootFiles/RecHD/com.apple.recovery.boot
defaults write /swap/.IABootFiles/RecHD/com.apple.recovery.boot/com.apple.Boot.plist "Kernel Flags" -string "$kflags"
defaults write /swap/.IABootFiles/RecHD/com.apple.recovery.boot/com.apple.Boot.plist "Boot Logo" -string "\BootLogo.png"
defaults write /swap/.IABootFiles/RecHD/com.apple.recovery.boot/com.apple.Boot.plist "Kernel Cache" -string "\com.apple.recovery.boot\kernelcache"
plutil -convert xml1 /swap/.IABootFiles/RecHD/com.apple.recovery.boot/com.apple.Boot.plist
	
### Making the recovery boot ###

chflags nouchg /swap/.IABootFiles/RecHD/com.apple.recovery.boot/boot.efi
rm /swap/.IABootFiles/RecHD/com.apple.recovery.boot/boot.efi
cp /usr/standalone/i386/boot.efi /swap/.IABootFiles/RecHD/com.apple.recovery.boot

### Copying/Creating some empty files used by MCPF res apps ###

if [ -f /swap/.IABootFiles/updSys.mcpfoption ]; then cp /swap/.IABootFiles/updSys.mcpfoption /swap/.IABootFiles/RecHD ; fi
if [ -f /swap/.IABootFiles/CLSys.mcpfoption ]; then cp /swap/.IABootFiles/CLSys.mcpfoption /swap/.IABootFiles/RecHD ; fi
if [ -f /swap/.IABootFiles/ocInst.mcpfoption ]; then cp /swap/.IABootFiles/ocInst.mcpfoption /swap/.IABootFiles/RecHD ; fi

#file_list=(*.mcpfoption
#);
#for file in "${file_list[@]}"; do
#	nbFile=$(find /swap/.IABootFiles/* -name "$file" ''-maxdepth 1'' | wc -l)
#	if (( $nbFile >= 1 )); then cp /swap/.IABootFiles/$file /swap/.IABootFiles/RecHD ; fi
#done

runMP=$(defaults read /swap/.IABootFiles/runVolumeInfo.mcpf.plist VolumeName)
echo "$runMP /mcpf/ end" > /swap/.IABootFiles/RecHD/rebootAfterInit.mcpf
	
### Set the RecHD partition's name for the Apple Boot Chooser (alt key at startup) ###

volName=$(defaults read /swap/.IABootFiles/destInfo.mcpf.plist VolumeName)
bless --folder /swap/.IABootFiles/RecHD/com.apple.recovery.boot --file /swap/.IABootFiles/RecHD/com.apple.recovery.boot/boot.efi -label "$volName Recovery" --setBoot
	
### Unmount the RecHD ###

/System/Installation/CDIS/osascript /swap/.IABootFiles/diskimgUnmount.scpt "/swap/.IABootFiles/RecHD"
if [ -d /swap/.IABootFiles/RecHD ] && [ ! -f /swap/.IABootFiles/RecHD/com.apple.recovery.boot/boot.efi ]; then rm -R /swap/.IABootFiles/RecHD ; fi

### END ###

bless --unbless /swap/.IABootFiles
mv /swap/.IABootFiles/boot.efi /swap/.IABootFiles/boot.efi.bak

###

exit 0