#!/bin/bash

# Graphics.mpkg
# MCPF Graphics Hack & Kexts Installer Script. Copyrights (c) @MLforAll. All rights reserved.
# Install Script v3.1
# Last revision 25/02/17

### Output ###

echo ""
echo "Script: Graphics"
echo ""

### Get base dir ###

if [ -f "$(pwd)/$(basename "$0")" ]; then
	sdir=$(pwd)
else
	sdir=$(dirname "$0")
fi

graphicsDatasLocation="$sdir/GraphicsDatas"
atinvidiaGraphicsKextsLocation="$graphicsDatasLocation/ATINVIDIAGraphicsKexts"
gma32KextsLocation="$graphicsDatasLocation/GMA32Kexts"
gma64KextsLocation="$graphicsDatasLocation/GMA64Kexts"

### Get base extract dir ###

#if [ "$1" = "/Volumes/BS" ]; then
#	baseExtractDir="/tmp"
#else
#	baseExtractDir="$1"
#fi

### Get OS X version ###

ver=$(defaults read "$1/System/Library/CoreServices/SystemVersion.plist" ProductVersion)

case "$ver" in
10.8*)
	osxinst="ml"
	gfxFolder="ML"
	;;
10.9*)
	osxinst="mav"
	gfxFolder="Mav"
	;;
10.10*)
	osxinst="yos"
	gfxFolder="Yos"
	;;
10.11*)
	osxinst="ec"
	gfxFolder="EC"
	;;
10.12*)
	osxinst="sierra"
	gfxFolder="Sierra"
	;;
esac

### GMA-equipped Mac ? ###

model=$(sysctl hw.model)

if [ "$model" == "hw.model: MacBook2,1" -o "$model" == "hw.model: MacBook3,1" -o "$model" == "hw.model: MacBook4,1" -o "$model" == "hw.model: iMac4,2" -o "$model" == "hw.model: iMac5,2" -o "$model" == "hw.model: Macmini1,1" -o "$model" == "hw.model: Macmini2,1" ]; then
	if [ ! -f "$1/Hack/32install.mcpfoption" ]; then
		gfxSubFolder="GMA"
	else
		gfxSubFolder="MOF" # in case of a 32bit hack (default) on a GMA-equipped computer, 10.7.5 kexts are used
	fi
else
	gfxSubFolder="MOF"
fi

### Replacing Frameworks ###

if [ ! -f "$1/Hack/instWithoutGFXexploit.mcpfoption" ] && [ "$osxinst" == "ml" ]; then

	cp -R -p "$graphicsDatasLocation/GFX_$gfxSubFolder_$gfxFolder/OpenGL.framework" "$1/System/Library/Frameworks/OpenGL.framework_"
	cp -R -p "$graphicsDatasLocation/GFX_$gfxSubFolder_$gfxFolder/OpenCL.framework" "$1/System/Library/Frameworks/OpenCL.framework_"

	if [ -d "$1/System/Library/Frameworks/OpenGL.framework_" ] && [ -d "$1/System/Library/Frameworks/OpenCL.framework_" ]; then
		rm -R "$1/System/Library/Frameworks/OpenGL.framework"
		rm -R "$1/System/Library/Frameworks/OpenCL.framework"
		mv "$1/System/Library/Frameworks/OpenGL.framework_" "$1/System/Library/Frameworks/OpenGL.framework"
		mv "$1/System/Library/Frameworks/OpenCL.framework_" "$1/System/Library/Frameworks/OpenCL.framework"
	else
		echo "Newly copied frameworks aren't there! The safety system will skip the GFX framework replacement to avoid damages to the computer!"
	fi
fi

### Managing Kexts ###

if [ ! -f "$1/Hack/instWithoutGFXkexts.mcpfoption" ]; then

	cd "$1/System/Library/Extensions"
	for k in ATI1300Controller.kext ATI1600Controller.kext ATI1900Controller.kext ATI2400Controller.kext ATI2600Controller.kext ATI3800Controller.kext ATI4600Controller.kext ATI4800Controller.kext ATI5000Controller.kext ATI6000Controller.kext ATIFramebuffer.kext ATIRadeonX1000.kext ATIRadeonX1000GA.plugin ATIRadeonX1000GLDriver.bundle ATIRadeonX1000VADriver.bundle ATIRadeonX2000.kext ATIRadeonX2000GA.plugin ATIRadeonX2000GLDriver.bundle ATIRadeonX2000VADriver.bundle ATIRadeonX3000.kext ATIRadeonX3000GA.plugin ATIRadeonX3000GLDriver.bundle ATIRadeonX3000VADriver.bundle ATISupport.kext GeForce.kext GeForce7xxx.kext GeForce7xxxGA.plugin GeForce7xxxGLDriver.bundle GeForce7xxxVADriver.bundle GeForceGA.plugin GeForceGLDriver.bundle GeForceVADriver.bundle NVDAGF100Hal.kext NVDAGK100Hal.kext NVDANV40HalG7xxx.kext NVDANV50Hal.kext NVDAResman.kext NVDAResmanG7xxx.kext NVSMU.kext AppleIntelGMA950.kext AppleIntelGMA950GLDriver.bundle AppleIntelGMA950VADriver.bundle AppleIntelGMA950GA.plugin AppleIntelIntegratedFramebuffer.kext AppleIntelGMAX3100.kext AppleIntelGMAX3100FB.kext AppleIntelGMAX3100GA.plugin AppleIntelGMAX3100GLDriver.bundle AppleIntelGMAX3100VADriver.bundle AMDFramebuffer.kext AMDSupport.kext
	do
		if [ -d "$k" ]; then rm -R "$k"; fi
	done
	
	if [ -f "$1/Hack/32install.mcpfoption" ]; then
		cp -R "$gma32KextsLocation"/* "$1/System/Library/Extensions"
	else
		cp -R "$atinvidiaGraphicsKextsLocation"/* "$1/System/Library/Extensions"
		cp -R "$gma64KextsLocation"/* "$1/System/Library/Extensions"
	fi
	
	chmod -R 755 "$1/System/Library/Extensions"
	chown -R root:wheel "$1/System/Library/Extensions"
	touch "$1/System/Library/Extensions"
	kextcache -f -update-volume "$1"

fi

### End of the script ###

echo ""
echo "-------------"
echo ""

exit 0
